/* GTK - The GIMP Toolkit
 * eggprintoperation.c: Print Operation
 * Copyright (C) 2005, Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "eggprintoperation-private.h"
#include "eggprintcontext-private.h"
#include "eggprintmarshal.h"
#include <cairo-pdf.h>

#define EGG_PRINT_OPERATION_GET_PRIVATE(obj)(G_TYPE_INSTANCE_GET_PRIVATE ((obj), EGG_TYPE_PRINT_OPERATION, EggPrintOperationPrivate))

enum {
  BEGIN_PRINT,
  REQUEST_PAGE_SETUP,
  DRAW_PAGE,
  END_PRINT,
  LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = { 0 };

G_DEFINE_TYPE (EggPrintOperation, egg_print_operation, G_TYPE_OBJECT)

static void
egg_print_operation_finalize (GObject *object)
{
  EggPrintOperation *print_operation = EGG_PRINT_OPERATION (object);

  g_object_unref (print_operation->priv->default_page_setup);
  g_object_unref (print_operation->priv->printer_settings);
  g_free (print_operation->priv->pdf_target);

  G_OBJECT_CLASS (egg_print_operation_parent_class)->finalize (object);
}

static void
egg_print_operation_init (EggPrintOperation *operation)
{
  operation->priv = EGG_PRINT_OPERATION_GET_PRIVATE (operation);

  operation->priv->default_page_setup = egg_page_setup_new ();
  operation->priv->printer_settings = egg_printer_settings_new ();
  operation->priv->nr_of_pages = EGG_PRINT_UNKNOWN_NR_OF_PAGES;
  operation->priv->use_full_page = FALSE;
  operation->priv->show_dialog = TRUE;
  operation->priv->pdf_target = NULL;
  operation->priv->unit = EGG_UNIT_PIXEL;
}

static void
egg_print_operation_class_init (EggPrintOperationClass *class)
{
  GObjectClass *gobject_class = (GObjectClass *)class;

  gobject_class->finalize = egg_print_operation_finalize;
  
  g_type_class_add_private (gobject_class, sizeof (EggPrintOperationPrivate));


  signals[BEGIN_PRINT] =
    g_signal_new ("begin_print",
		  G_TYPE_FROM_CLASS (gobject_class),
		  G_SIGNAL_RUN_LAST,
		  G_STRUCT_OFFSET (EggPrintOperationClass, begin_print),
		  NULL, NULL,
		  g_cclosure_marshal_VOID__OBJECT,
		  G_TYPE_NONE, 1, EGG_TYPE_PRINT_CONTEXT);
  signals[REQUEST_PAGE_SETUP] =
    g_signal_new ("request_page_setup",
		  G_TYPE_FROM_CLASS (gobject_class),
		  G_SIGNAL_RUN_LAST,
		  G_STRUCT_OFFSET (EggPrintOperationClass, request_page_setup),
		  NULL, NULL,
		  _egg_marshal_VOID__OBJECT_INT_OBJECT,
		  G_TYPE_NONE, 3,
		  EGG_TYPE_PRINT_CONTEXT,
		  G_TYPE_INT,
		  EGG_TYPE_PAGE_SETUP);
  signals[DRAW_PAGE] =
    g_signal_new ("draw_page",
		  G_TYPE_FROM_CLASS (gobject_class),
		  G_SIGNAL_RUN_LAST,
		  G_STRUCT_OFFSET (EggPrintOperationClass, draw_page),
		  NULL, NULL,
		  _egg_marshal_BOOL__OBJECT_INT,
		  G_TYPE_BOOLEAN, 2,
		  EGG_TYPE_PRINT_CONTEXT,
		  G_TYPE_INT);
  signals[END_PRINT] =
    g_signal_new ("end_print",
		  G_TYPE_FROM_CLASS (gobject_class),
		  G_SIGNAL_RUN_LAST,
		  G_STRUCT_OFFSET (EggPrintOperationClass, end_print),
		  NULL, NULL,
		  g_cclosure_marshal_VOID__OBJECT,
		  G_TYPE_NONE, 1, EGG_TYPE_PRINT_CONTEXT);
}

EggPrintOperation *
egg_print_operation_new ()
{
  EggPrintOperation *print_operation;

  print_operation = g_object_new (EGG_TYPE_PRINT_OPERATION, NULL);
  
  return print_operation;
}

void
egg_print_operation_set_default_page_setup (EggPrintOperation  *op,
					       EggPageSetup    *default_page_setup)
{
  g_return_if_fail (op != NULL);
  g_return_if_fail (default_page_setup != NULL);
  
  if (default_page_setup != op->priv->default_page_setup)
    {
      g_object_unref (op->priv->default_page_setup);
      op->priv->default_page_setup = g_object_ref (default_page_setup);
    }
}

EggPageSetup *
egg_print_operation_get_default_page_setup (EggPrintOperation  *op)
{
  g_return_val_if_fail (op != NULL, NULL);

  return op->priv->default_page_setup;
}

void
egg_print_operation_set_printer_settings (EggPrintOperation  *op,
					  EggPrinterSettings *printer_settings)
{
  g_return_if_fail (op != NULL);
  g_return_if_fail (printer_settings != NULL);
  
  if (printer_settings != op->priv->printer_settings)
    {
      g_object_unref (op->priv->printer_settings);
      op->priv->printer_settings = egg_printer_settings_copy (printer_settings);
    }
}

EggPrinterSettings *
egg_print_operation_get_printer_settings (EggPrintOperation  *op)
{
  g_return_val_if_fail (op != NULL, NULL);

  return op->priv->printer_settings;
}

void
egg_print_operation_set_nr_of_pages (EggPrintOperation  *op,
				     int                 n_pages)
{
  g_return_if_fail (op != NULL);

  op->priv->nr_of_pages = n_pages;
}

void
egg_print_operation_set_use_full_page (EggPrintOperation  *op,
				       gboolean            full_page)
{
  g_return_if_fail (op != NULL);

  op->priv->use_full_page = full_page;
}

void
egg_print_operation_set_unit (EggPrintOperation  *op,
			      EggUnit             unit)
{
  op->priv->unit = unit;
}

void
egg_print_operation_set_show_dialog (EggPrintOperation  *op,
				     gboolean            show_dialog)
{
  g_return_if_fail (op != NULL);

  op->priv->show_dialog = show_dialog;
}

void
egg_print_operation_set_pdf_target (EggPrintOperation  *op,
				    const char *        filename)
{
  g_return_if_fail (op != NULL);

  op->priv->pdf_target = g_strdup (filename);
}

static gboolean
run_print_dialog (EggPrintOperation  *op)
{
  if (op->priv->pdf_target != NULL) {
    double width, height;
    /* This will be overwritten later by the non-default size, but
       we need to pass some size: */
    
    width = egg_page_setup_get_paper_width (op->priv->default_page_setup, EGG_UNIT_POINTS);
    height = egg_page_setup_get_paper_height (op->priv->default_page_setup, EGG_UNIT_POINTS);
    
    op->priv->surface = cairo_pdf_surface_create (op->priv->pdf_target,
						  width, height);
    /* TODO: DPI from settings object? */
    cairo_pdf_surface_set_dpi (op->priv->surface, 300, 300);

    op->priv->dpi_x = 72;
    op->priv->dpi_y = 72;
    
    return TRUE; /* No dialog for pdf writing */
  }


  /* Open print dialog */
  /* set print settings on dialog */
  /* run dialog, if show_dialog */
  /* extract print settings from dialog */
  /* if print selected, return TRUE, else FALSE */

  return TRUE;
}

void
egg_print_operation_run (EggPrintOperation  *op)
{
  int page;
  EggPageSetup *page_setup;
  EggPrintContext *print_context;
  gboolean draw_page_res;
  cairo_t *cr;
  
  if (!run_print_dialog (op))
    return;
  
  print_context = _egg_print_context_new (op);

  g_signal_emit (op, signals[BEGIN_PRINT], 0, print_context);

  page = 0;
  while (op->priv->nr_of_pages == EGG_PRINT_UNKNOWN_NR_OF_PAGES ||
	 page < op->priv->nr_of_pages)
    {
      page_setup = egg_page_setup_copy (op->priv->default_page_setup);
      g_signal_emit (op, signals[REQUEST_PAGE_SETUP], 0, print_context, page, page_setup);
      _egg_print_context_set_page_setup (print_context, page_setup);
      g_object_unref (page_setup);

      cr = egg_print_context_get_cairo (print_context);
      
      /* TODO: set cairo_t page setup from page_setup, needs new API */
      
      cairo_save (cr);

      if (!op->priv->use_full_page)
	_egg_print_context_translate_into_margin (print_context);
      
      g_signal_emit (op, signals[DRAW_PAGE], 0, 
		     print_context, page,
		     &draw_page_res);

      if (!draw_page_res && op->priv->nr_of_pages == EGG_PRINT_UNKNOWN_NR_OF_PAGES)
	op->priv->nr_of_pages = page;

      if (draw_page_res)
	cairo_show_page (cr);
      
      cairo_restore (cr);

      page++;
    }
  
  g_signal_emit (op, signals[END_PRINT], 0, print_context);

  g_object_unref (print_context);

  cairo_surface_destroy (op->priv->surface);
  op->priv->surface = NULL;
}
